
type ctxt = 
    {
      dry_run: bool;
      verbose: bool;
      target: string; (* Target topdir. *)

      (* Where to look for extra stublibs. *)
      std_stublib_dirs: string list;

      (* File Hierarchy Standard. *) 
      bindir: string;
      logdir: string;
      mkdir_logdir: bool; (* Do we need to mkdir the logdir, we don't create it if defined externaly. *)
      etcdir: string;
      moddir: string; (* Where to store modules (.cmo, .cma). *)
      rundir: string; (* Where to put command pipe. *)
      libdir: string;
      ocamldir: string; (* Where to store findlib libraries. *)
      stubdir: string; (* Where to store stublbis. *) 
      commandpipe: string;
      mimefile: string;
    }

let ctxt_default ?(dry_run=false) ?(verbose=false) tgt = 
  let libdir = "lib" in
  let ocamldir = Filename.concat libdir "ocaml" in
  let rundir = "run" in
  let etcdir = "etc" in
    {
      dry_run = dry_run;
      verbose = verbose;
      target = tgt;

      std_stublib_dirs = [];

      bindir = "bin";
      logdir = "log";
      mkdir_logdir = true;
      etcdir = etcdir;
      moddir = "mod";
      rundir = rundir;
      libdir = libdir;
      ocamldir = ocamldir;
      stubdir = Filename.concat ocamldir "stublibs";
      commandpipe = Filename.concat rundir "ocsigen_command";
      mimefile = FilePath.make_filename [etcdir; "ocsigen"; "mime.types"];
    }

let info ~ctxt fmt = 
  if ctxt.verbose then
    begin
      Printf.printf "I: ";
      Printf.kfprintf
        (fun chn -> Printf.fprintf chn "\n%!")
        stdout
        fmt
    end
  else
    begin
      Printf.ifprintf stdout fmt
    end

let warning ~ctxt fmt = 
  Printf.eprintf "W: ";
  Printf.kfprintf 
    (fun chn -> Printf.fprintf chn "\n%!")
    stderr
    fmt

let install ~ctxt ?tgt_rel ?chmod (src, tgt_tl) = 
  let tgt_fn = 
    match tgt_rel with 
      | Some tgt_rel ->
          FilePath.make_filename [ctxt.target; tgt_rel; tgt_tl]
      | None ->
          FilePath.concat ctxt.target tgt_tl
  in
  let chmod = 
    match chmod with 
      | Some v -> v
      | None ->
          let org_chmod = (Unix.stat src).Unix.st_perm in
            (* Don't allow 'other' to do anything *)
            0o770 land org_chmod
  in
    info ~ctxt "install '%s' -> '%s'" src tgt_fn;
    if not ctxt.dry_run then
      begin
        FileUtil.mkdir ~parent:true (FilePath.dirname tgt_fn);
        FileUtil.cp [src] tgt_fn;
        Unix.chmod tgt_fn chmod
      end

let info_mkdir ~ctxt dn = 
  let dn =
    FilePath.concat ctxt.target dn
  in
    info ~ctxt "mkdir '%s'" dn;
    if not ctxt.dry_run then
      FileUtil.mkdir ~parent:true dn

let create_file ~ctxt ?chmod fn f = 
  let fn' =
    FilePath.concat ctxt.target fn
  in
    info_mkdir ~ctxt (FilePath.dirname fn);
    info ~ctxt "Creating file '%s'" fn';
    if not ctxt.dry_run then
      begin
        let chn =
          open_out fn'
        in
          f chn;
          close_out chn;
          match chmod with 
            | Some n ->
                Unix.chmod fn' n
            | None ->
                ()
      end
    else
      begin
        (* For the sake of testing, we still create the content. *)
        let tmpfn, chn_out = Filename.open_temp_file "ocsigen-bundler" ".txt" in
        let clean () = close_out chn_out; Sys.remove tmpfn in
          try 
            f chn_out;
            clean ()
          with e ->
            clean ();
            raise e
      end

let eval_ocaml_script ~descr script = 
  let tmpfn, chn_out = Filename.open_temp_file "ocsigen-bundler" ".ml" in
  let clean () = Sys.remove tmpfn in
  let () =
    output_string chn_out script;
    close_out chn_out
  in
  let chn_in = 
    Unix.open_process_in 
      (Printf.sprintf "ocaml %s" (Filename.quote tmpfn))
  in        
  let res = input_line chn_in in
    match Unix.close_process_in chn_in with 
      | Unix.WEXITED 0 -> 
          clean ();
          res
      | Unix.WEXITED n 
      | Unix.WSIGNALED n 
      | Unix.WSTOPPED n ->
          clean ();
          failwith 
            (Printf.sprintf 
               "The process 'ocaml' runnning %s stopped with error %d"
               descr n)
